#!/bin/bash
# Parse a relaxation file  and create postsript plots for a given month
# Stand-alone version, template included in script

usage="
Routine plots forcing fields from the relaxation files relax.XXX.[ab] files
Usage: $(basename $0) [-i incr] [-c center] [-p plottype] month  var
 Where:  month  is the month to plot
         var    is the variable to plot ( specifying var as temp plots data from relax.temp.[ab])
 Optional: 
         -i incr      where incr is contour interval to use
         -c center    where center is the center contour interval (ignored if incr is not given)
         -p plottype  can be set to psp, psl or x11
                  psp is for postscript output (portrait)
                  psl is for postscript output (landscape)
                  x11 is for screen output

The following example shows how to plot air temperature (from forcing.airtmp.[ab] files)
for January, using a contour interval of .5 and a contour interval center of 5 deg C.
The output is displayed onscreen:
  plotrelax -c 5 -i .5 -p x11 1 temp
The following example shows how to temperature using autogenerated contours
  plotrelax 1 temp "

# Get command line args
incr=
center=
scale=1
plottype=x11
while getopts "i:c:s:p:" options; do
     case $options in
        i ) incr=$OPTARG ;;
        c ) center=$OPTARG ;;
        p ) plottype=$OPTARG ;;
        * ) echo -e "$usage"
        exit 1;;
     esac
done
#echo incr=$incr
#echo center=$center
#echo scale=$scale
#echo plottype=$plottype
shift $(($OPTIND - 1))
#echo $@

if [ $# -lt 2 ] ; then
  echo -e "$usage" 
  exit 1
fi

month=$1
var=$2
month2=$(echo 0$month | tail -c3)


# Parse hycom blkdat
[ ! -f  blkdat.input ]  && { echo "Input file blkdat.input does not exist "; exit 1 ; }
IDM=$(cat blkdat.input | grep idm | sed  "s/^[ ]*//" | cut -f1 )
JDM=$(cat blkdat.input | grep jdm | sed  "s/^[ ]*//" | cut -f1 )
KDM=$(cat blkdat.input | grep kdm | sed  "s/^[ ]*//" | cut -f1 )
echo IDM $IDM
echo JDM $JDM
echo KDM $KDM

# Name check
echo $var
[ "$(echo $var | cut -c1-3)" == "sal" ] && variable=Salinity
[ "$(echo $var | cut -c1-3)" == "tem" ] && variable=Temperature
[ "$(echo $var | cut -c1-3)" == "int" ] && variable=Interface

# Plot type check
[ "$plottype" != "x11" -a "$plottype" != "psp" -a "$plottype" != "psl" ] && \
   { echo "Supply plot type as x11, psp or psl" ; exit 1 ;}

# Do input files exist?
FILE="relax.$var.a"
FILEb="relax.$var.b"
[ ! -f  $FILE ]  && { echo "Relaxation file $FILE does not exist..." ; exit 1 ;}
[ ! -f  $FILEb ] && { echo "Relaxation file $FILEb does not exist..." ; exit 1 ;}

# Test for Routinedir
if [ "${HYCOM_ALL}" == "" ]; then
   echo "Environment variable HYCOM_ALL is empty"
   exit 1
fi

# Test for main routine
FP=${HYCOM_ALL}/plot/src/fp_$plottype 
if [ ! -f $FP ]; then
   echo "Cant find routine $FP"
   exit 1
fi

# Infiledir
#TEMPLATE="/home/nersc/knutali/Progs/MSCProgs/scripts/relax.IN.Template"
TEMPLATE="FILE           
ATLb2.00
IDM     'idm   ' = longitudinal array size
JDM     'jdm   ' = latitudinal  array size
  1     'nperfr' = number of horizontal plots per frame
  0     'lalolb' = spacing of latitude/longitude labels
  0     'lalogr' = spacing of latitude/longitude grid over land (<0 land+sea)
  1     'loclab' = location of the contour label (1=upr,2=lowr,3=lowl,4=upl)
CLC     'locbar' = location of the color bar     (1[0-4]=vert,2[0-4]=horiz)
PLT     'kpalet' = palete (0=none,1=pastel,2=sst,3=gaudy,4=2tone,5=fc,6=ifc)
  1     'iorign' = i-origin of plotted subregion
  1     'jorign' = j-origin of plotted subregion
  0     'idmp  ' = i-extent of plotted subregion (<=idm; 0 implies idm)
  0     'jdmp  ' = j-extent of plotted subregion (<=jdm; 0 implies jdm)"

# For Color bar location 
CLC=13

# For color palette
PLT=5


#cat $TEMPLATE | sed "s/IDM/$IDM/" | sed "s/JDM/$JDM/" | sed "s/FILE/$FILE/" | \
#                sed "s/PLT/$PLT/" | sed "s/CLC/$CLC/" > relax_$var.IN

echo -e "$TEMPLATE" | sed "s/IDM/$IDM/" | sed "s/JDM/$JDM/" | sed "s/FILE/$FILE/" | \
                    sed "s/PLT/$PLT/" | sed "s/CLC/$CLC/" > relax_$var.IN



# Skip to correct month record
let mm1=$month-1
let firstrec=$KDM*$mm1
irec=1
while [ $irec -le $KDM ] ;
do
   let irec2=$firstrec+$irec
   echo " $irec2	'nrec  '" >> relax_$var.IN
   echo " Relaxation $variable of layer $irec for Month $month"        >> relax_$var.IN

   if [ "$(echo $var | cut -c1-3)" != "int" ] ; then
      echo "  1	'qscale'" >> relax_$var.IN
   else
      echo "0.0001	'qscale'" >> relax_$var.IN
   fi

   if [ "$incr" != "" -a "$center" != ""  ] ; then
      echo " $incr	'qq    '" >> relax_$var.IN
      echo "$center	'center'" >> relax_$var.IN
   else
      echo "   0.0	'qq    '" >> relax_$var.IN
   fi

   let irec=$irec+1
done
echo " -1	'nrec  '" >> relax_$var.IN
   




#Generate salinity and temperature plots
$FP < relax_$var.IN 
if [ "$plottype" != "x11" ] ; then
   mv gmeta1.ps relax_${var}_m$month2.ps
   echo "Postscript file is relax_${var}_m$month2.ps"
fi


